import React, { useEffect, useRef, useState, forwardRef, useImperativeHandle } from 'react';
import { GraphType, LinkType, NodeType, NodeTypeD3 } from './types';
import { NLPixi } from './components/NLPixi';
import { parseQueryResult } from './components/query2NL';
import { useImmer } from 'use-immer';
import { ML, setShortestPathSource, setShortestPathTarget } from '../../../data-access/store/mlSlice';
import { Layouts } from '../../../graph-layout/types';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import { EntityPill } from '@graphpolaris/shared/lib/components/pills/Pill';
import { nodeColorHex } from './components/utils';
import { Node } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { IPointData } from 'pixi.js';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';

export interface NodeLinkVisHandle {
  exportImageInternal: () => void;
}

export interface NodelinkVisProps {
  id: string;
  name: string;
  layout: string;
  showPopUpOnHover: boolean;
  shapes: {
    similar: boolean;
    shape: 'circle' | 'rectangle';
    shapeMap: { [id: string]: 'circle' | 'rectangle' } | undefined;
  };
  edges: {
    width: {
      similar: boolean;
      width: number;
    };
  };
  nodeList: string[];
}

const settings: NodelinkVisProps = {
  id: 'NodeLinkVis',
  name: 'NodeLinkVis',
  layout: Layouts.FORCEATLAS2WEBWORKER as string,
  showPopUpOnHover: false,
  shapes: {
    similar: true,
    shape: 'circle',
    shapeMap: undefined,
  },
  edges: {
    width: { similar: true, width: 0.8 },
  },
  nodeList: [],
};

const NodeLinkVis = forwardRef<NodeLinkVisHandle, VisualizationPropTypes<NodelinkVisProps>>(
  ({ data, ml, dispatch, settings, handleSelect }, refExternal) => {
    const ref = useRef<HTMLDivElement>(null);
    const nlPixiRef = useRef<any>(null);
    const [graph, setGraph] = useImmer<GraphType | undefined>(undefined);
    const [highlightNodes, setHighlightNodes] = useState<NodeType[]>([]);
    const [highlightedLinks, setHighlightedLinks] = useState<LinkType[]>([]);

    useEffect(() => {
      if (data) {
        setGraph(
          parseQueryResult(data, ml, {
            defaultX: (ref.current?.clientWidth || 1000) / 2,
            defaultY: (ref.current?.clientHeight || 1000) / 2,
          }),
        );
      }
    }, [data, ml]);

    const onClickedNode = (event?: { node: NodeTypeD3; pos: IPointData }, ml?: ML) => {
      if (graph) {
        if (!event?.node) {
          handleSelect();
          return;
        }

        const node = event.node;
        const nodeMeta = graph.nodes[node._id];
        handleSelect({ nodes: [nodeMeta as Node] });

        if (ml && ml.shortestPath.enabled) {
          setGraph((draft) => {
            let _node = draft?.nodes[node._id];
            if (!_node) return draft;

            if (!ml.shortestPath.srcNode) {
              _node.isShortestPathSource = true;
              dispatch(setShortestPathSource(node._id));
            } else if (ml.shortestPath.srcNode === node._id) {
              _node.isShortestPathSource = false;
              dispatch(setShortestPathSource(undefined));
            } else if (!ml.shortestPath.trtNode) {
              _node.isShortestPathTarget = true;
              dispatch(setShortestPathTarget(node._id));
            } else if (ml.shortestPath.trtNode === node._id) {
              _node.isShortestPathTarget = false;
              dispatch(setShortestPathTarget(undefined));
            } else {
              _node.isShortestPathSource = true;
              _node.isShortestPathTarget = false;
              dispatch(setShortestPathSource(node._id));
              dispatch(setShortestPathTarget(undefined));
            }
            return draft;
          });
        }
      }
    };

    const exportImageInternal = () => {
      nlPixiRef.current.exportImage();
    };

    useImperativeHandle(refExternal, () => ({
      exportImageInternal,
    }));

    if (!graph) return null;

    return (
      <NLPixi
        ref={nlPixiRef}
        graph={graph}
        configuration={settings}
        highlightNodes={highlightNodes}
        highlightedLinks={highlightedLinks}
        onClick={(event) => {
          onClickedNode(event, ml);
        }}
        layoutAlgorithm={settings.layout}
        showPopupsOnHover={settings.showPopUpOnHover}
      />
    );
  },
);

const NodelinkSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<NodelinkVisProps>) => {
  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      updateSettings({ nodeList: graphMetadata.nodes.labels });
    }
  }, [graphMetadata]);

  if (!settings.nodeList) return null;

  return (
    <SettingsContainer>
      <div className="mb-4 text-xs">
        <h1 className="font-bold">General</h1>
        <div className="m-1 flex flex-col space-y-2 mb-2">
          <h4 className="font-semibold">Nodes Labels:</h4>
          {settings.nodeList.map((item, index) => (
            <div className="flex m-1 items-center" key={item}>
              <div className="w-3/4 mr-6">
                <EntityPill title={item} />
              </div>
              <div className="w-1/2">
                <div className={`h-5 w-5 border-2 border-sec-300`} style={{ backgroundColor: nodeColorHex(index + 1) }}></div>
              </div>
            </div>
          ))}
        </div>
        <Input
          type="dropdown"
          label="Layout"
          value={settings.layout}
          options={Object.values(Layouts) as string[]}
          onChange={(val) => updateSettings({ layout: val as string })}
        />
        <Input
          type="boolean"
          label="Show pop-up on hover"
          value={settings.showPopUpOnHover}
          onChange={(val) => updateSettings({ showPopUpOnHover: val })}
        />
      </div>

      <div className="mb-4">
        <h1 className="font-bold">Nodes</h1>

        <div>
          <span className="text-xs font-semibold">Shape</span>
          <Input
            type="boolean"
            label="Common shape?"
            value={settings.shapes.similar}
            onChange={(val) => updateSettings({ shapes: { ...settings.shapes, similar: val } })}
          />
          {settings.shapes.similar ? (
            <Input
              type="dropdown"
              label="Shape"
              value={settings.shapes.shape}
              options={[{ circle: 'Circle' }, { rectangle: 'Square' }]}
              onChange={(val) => updateSettings({ shapes: { ...settings.shapes, shape: val as any } })}
            />
          ) : (
            <span>Map shapes to labels (to be implemented)</span>
          )}
        </div>

        <div>
          <span className="text-xs font-semibold">Color</span>
        </div>
      </div>

      <div>
        <h1 className="font-bold">Edges</h1>
        <div>
          <span className="text-xs font-semibold">Edge width</span>
          <Input
            type="boolean"
            label="Common width"
            value={settings.edges.width.similar}
            onChange={(val) => updateSettings({ edges: { ...settings.edges, width: { ...settings.edges.width, similar: val } } })}
          />
          <Input
            type="slider"
            label="Width"
            value={settings.edges.width.width}
            onChange={(val) => updateSettings({ edges: { ...settings.edges, width: { ...settings.edges.width, width: val } } })}
            min={0.1}
            max={4}
            step={0.1}
          />
        </div>
      </div>
    </SettingsContainer>
  );
};
const nodeLinkVisRef = React.createRef<{ exportImageInternal: () => void }>();

export const NodeLinkComponent: VISComponentType<NodelinkVisProps> = {
  displayName: 'NodeLinkVis',
  description: 'General Patterns and Connections',
  component: React.forwardRef((props: VisualizationPropTypes<NodelinkVisProps>, ref) => <NodeLinkVis {...props} ref={nodeLinkVisRef} />),
  settingsComponent: NodelinkSettings,
  settings: settings,
  exportImage: () => {
    if (nodeLinkVisRef.current) {
      nodeLinkVisRef.current.exportImageInternal();
    } else {
      console.error('NodeLink reference is not set.');
    }
  },
};

export default NodeLinkComponent;
